/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include <CConnector.h>

#include <CComboBox.h>
#include <CNameComboBox.h>

#include <CPropertyList.h>
#include <CTracker.h>
#include <CTrackManager.h>
#include <CApp.h>
#include <CAppDialog.h>

class CPropertyName : public CProperty
{
public:
	CPropertyName(IDispenser* pDispenser, CNameComboBox* pNameComboBox, CNameListItem* pNameListItem)
	: CProperty(pDispenser)
	{
		m_pNameComboBox = pNameComboBox;
		m_pNameListItem = pNameListItem;
	}

	virtual ~CPropertyName() {}

	const char* name(void) {return "Name";}
	virtual ePropertyType type(void) {return PROPERTY_TEXT;}
	virtual const char* description(void){return "The name used in code to identify\nthis object";}

	void set(FXWindow* pWindow, FXString strValue)
	{
		m_pNameListItem->setText(strValue);
		m_pNameComboBox->setText(strValue);
	}

	FXString get(FXWindow* pWindow)
	{
		return m_pNameListItem->getText();
	}

private:
	CNameComboBox* m_pNameComboBox;
	CNameListItem* m_pNameListItem;
};

CConnector::CConnector()
{
	pTrackManager = 0;
	pDispenser    = 0;
	pProperty     = 0;
	pWidget       = 0;
	pProperties   = 0;
	pDescription  = 0;
	pButton       = 0;
	pNames        = 0;
	bDoubleClick  = FALSE;
}

FXWindow* CConnector::addWidget(FXWindow* pParent, FXbool bCreate, FXbool bSetCurrent, FXshort x, FXshort y, FXshort w, FXshort h)
{
	// We are now going to create a copy of the IDispenser*
	// for each instance of the widget to support properties that are not
	// associated with a window.
	//
	// For example: If you create two textboxes on a dialog and you move 
	// backward and forward between the Text Property. You'll see that 
	// each instance of the Text Box has its own value in the Text property.
	// This is because the per instance data is stored in the FXTextField
	// window itself.
	//
	// However, the Name property cannot store its per instance data in the
	// FXWindow since there is no equivalent in the implementation of FXWindow.
	// One option would be to use the void* UserData associated with each
	// FXWindow. Unfortunately, that's not very extensible.

	// When the window is destroyed -- either the application
	// is closing or the user wants to delete the widget -- then 
	// CTrackManager will need to delete the associated IDispenser*

	IDispenser* pOldDispenser = 0;
	IDispenser* pNewDispenser = pDispenser->clone();

	pWidget = pNewDispenser->createWidget((FXComposite*)pParent, 0, 0);

	if ( x==0 && y==0 && w==0 && h==0 )
	{
		pWidget->setLayoutHints(LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y);
	}

	if ( w < pNewDispenser->getMinimumWidth() )
		w = pNewDispenser->getMinimumWidth();

	if ( w > pNewDispenser->getMaximumWidth() )
		w = pNewDispenser->getMaximumWidth();

	if ( h < pNewDispenser->getMinimumHeight() )
		h = pNewDispenser->getMinimumHeight();

	if ( h > pNewDispenser->getMaximumHeight() )
		h = pNewDispenser->getMaximumHeight();

	// CMW 04.11.99 If this function is called before the create stage (as CMainFrame does
	//				as it constructs the main GUI) then the caller must set bCreate to FALSE.
	//              If the GUI has already been created then setting bCreate to TRUE will
	//				display the widget
	if ( bCreate )
	{
		pWidget->create();
		pWidget->position(x, y, w, h);
		pWidget->enable();
		pWidget->show();
	}

	pOldDispenser = pDispenser;
	pDispenser    = pNewDispenser;
	pProperty     = 0;

	releaseButtons();

	// Manufacture a widget name
	char sz[5+1] = "";
	sprintf(sz, "%i", application->getDispenserCount(pNewDispenser));
	FXString strName = pNewDispenser->getWidgetName() + FXString(sz);

	// Set the widget name in the Name combo box
	CNameListItem* p = new CNameListItem( *this, strName, pNewDispenser->getWidgetName() );
	FXint index = pNames->appendItem(p);

	// Add the name property
	pNewDispenser->addProperty(new CPropertyName(pNewDispenser, pNames, p));

//	// Set the CConnector back pointer for each property
	// Set the widget name in the widget itself - if it supports a Text property
	FXint nCount = pNewDispenser->getPropertyCount();
	for (FXint i = 0; i < nCount; i++)
	{
		IProperty* pProperty = pNewDispenser->getProperty(i);

		if ( !strcmp(pProperty->name(), "Text") )
		{
			pProperty->set(pWidget, strName);
		}
	}

	if ( bSetCurrent )
	{
		if ( pProperties )
		{
			pProperties->setup(*this);
		}

		pNames->setCurrentItem(index);
	}

	application->addWidget(*this);

	// Now that CApp::addWidget has copied this CConnector
	// we need to restore the original pDispenser
	pDispenser = pOldDispenser;

	return pWidget;
}

void CConnector::select(FXbool bMultiple)
{
	pTrackManager->recalc();
	pTrackManager->select(*this, bMultiple);
	pProperties->setup(*this, bMultiple);
	pNames->setCurrentItem(*this, bMultiple);
}

void CConnector::update(const FXchar* pszName)
{
	IProperty* pProperty = &pDispenser->getProperty(pszName);
	FXString strText = pProperty->get(pWidget);
	pProperty->set(pWidget, strText);
}

void CConnector::remove(void)
{
	pNames->removeItem(*this);
}

void CConnector::releaseButtons(void)
{
	if ( pButton )
	{
		pButton->setState(STATE_UP);
	}
}

void CConnector::set(FXString strValue)
{
	FXint nCount = pTrackManager->getTrackerCount();

	if ( nCount)
	{
		for ( FXint i = 0; i < nCount; i++ )
		{
			CConnector* pConnector = pTrackManager->getConnector(i);

			if ( !pConnector )
				continue;

			pProperty->validate(this, strValue);

			pProperty->set(pConnector->pWidget, strValue);

			if ( pConnector->pWidget->getWidth() < pDispenser->getMinimumWidth() )
				pConnector->pWidget->setWidth(pDispenser->getMinimumWidth());

			if ( pConnector->pWidget->getWidth() > pDispenser->getMaximumWidth() )
				pConnector->pWidget->setWidth(pDispenser->getMaximumWidth());

			if ( pConnector->pWidget->getHeight() < pDispenser->getMinimumHeight() )
				pConnector->pWidget->setHeight(pDispenser->getMinimumHeight());

			if ( pConnector->pWidget->getHeight() > pDispenser->getMaximumHeight() )
				pConnector->pWidget->setHeight(pDispenser->getMaximumHeight());
		}
	}
	else
	{
		pProperty->validate(this, strValue);
		pProperty->set(pWidget, strValue);
	}

	pTrackManager->recalc();

	pTrackManager->clearGrabbers();
	application->setDirty();
}

FXString CConnector::get(void)
{
	pProperty->validate(this);
	return pProperty->get(pWidget);
}

void CConnector::refreshProperty(void)
{
	pProperties->refreshProperty();
}

void CConnector::close(void)
{
	pProperties->close();
}

void CConnector::setDirty(void)
{
	application->setDirty();
}
