/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

/*
 *  CAppDialog overrides CApp and provides application wide functionality.
 *  Specifically CAppDialog manages a list of active widgets and dispensers,
 *  the dirty flag for the app and redirects key events to the form painter.
 *
 *  TODO: It would make sense for some of this functionality to be in CApp.
 */

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include <CApp.h>
#include <CFile.h>
#include <CGarbage.h>

#include "CTracker.h"
#include "CTrackManager.h"
#include "CAppDialog.h"
#include "CMainFrame.h"

#include <CVariant.h>

CAppDialog* application;

const FXchar *patterns[]=
{
	"Dialog Files",  "*.dlg",
	"All Files",     "*",
	NULL
};

FXDEFMAP(CAppDialog) CAppDialogMap[]=
{
	FXMAPFUNC(SEL_COMMAND,FXApp::ID_QUIT,CAppDialog::onCmdQuit),
	FXMAPFUNC(SEL_COMMAND,CAppDialog::IDM_FILENEW,CAppDialog::onFileNew),
	FXMAPFUNC(SEL_COMMAND,CAppDialog::IDM_FILEOPEN,CAppDialog::onFileOpen),
	FXMAPFUNC(SEL_COMMAND,CAppDialog::IDM_FILESAVE,CAppDialog::onFileSave),
	FXMAPFUNC(SEL_COMMAND,CAppDialog::IDM_FILESAVEAS,CAppDialog::onFileSaveAs),
	FXMAPFUNC(SEL_UPDATE, CAppDialog::IDM_FILENEW,CAppDialog::onUpdateFileNew)
};

FXIMPLEMENT(CAppDialog, CApp, CAppDialogMap, ARRAYNUMBER(CAppDialogMap)) 

CAppDialog::CAppDialog(const FXchar* name, const FXchar* vendor) 
: CApp(name, vendor)
{
	m_pAppFrame = NULL;
	m_pFrame = NULL;
	m_pPane = NULL;

	m_bDirty = FALSE;
	m_bRunning = TRUE;
} 

CAppDialog::~CAppDialog()
{
	// AppPane
	delete m_Widgets[0]->pDispenser;
	delete m_Widgets[0];

	// Dialog
	delete m_Widgets[1]->pDispenser;
	delete m_Widgets[1];

	// DialogPane
	delete m_Widgets[2];

	clearWidgets(FALSE);

	while ( m_Widgets.no() ) 
	{
		m_Widgets.remove(0);
	} 
}

void CAppDialog::refreshWidgetTree(void)
{
	for ( FXint i = 0; i < m_Widgets.no(); i++ ) 
	{
		m_Widgets[i]->pWidget->recalc();
	}
	forceRefresh();
}

void CAppDialog::clearWidget(CConnector& connector)
{
	FXint i = 0;

	// If this window has children then delete them first
	for ( i = m_Widgets.no()-1; i >= 3; i-- ) 
	{
		FXWindow* parent = m_Widgets[i]->pWidget;

		do
		{
			parent = parent->getParent();

			if ( connector.pWidget == parent )
			{
				deleteWidget(i, TRUE);
			}

		}
		while ( parent );
	}
	for ( i = 3; i < m_Widgets.no(); i++ ) 
	{
		if ( m_Widgets[i]->pWidget == connector.pWidget )
		{
			deleteWidget(i, TRUE);
			return;
		}
	}
}

void CAppDialog::clearWidgets(FXbool bSelectDialog)
{
	for ( FXint i = 3; i < m_Widgets.no(); i++ ) 
	{
		deleteWidget(i, FALSE);
	}

	while ( m_Widgets.no() > 3 ) 
	{
		m_Widgets.remove(3);
	} 
}

void CAppDialog::deleteWidget(FXint nIndex, FXbool bForceDelete)
{
	FXWindow* pWnd = m_Widgets[nIndex]->pWidget;

	CConnector* pParent = 0;
	application->isValidWidget(pWnd->getParent(), &pParent);

	if	(
		bForceDelete ||
		(pParent && !pParent->pDispenser->widgetDeletesChildren())
		)
	{
		delete pWnd;
	}

	m_Widgets[nIndex]->remove();
	delete m_Widgets[nIndex]->pDispenser;
	delete m_Widgets[nIndex];
	m_Widgets.remove(nIndex);
}

void CAppDialog::addWidget(CConnector& connector)
{
	CConnector* pConnector = new CConnector;
	*pConnector = connector;
	m_Widgets.append(pConnector);
}

long CAppDialog::onCmdQuit(FXObject* o,FXSelector s,void* ptr)
{
	m_bRunning = FALSE;

	if ( m_bDirty )
	{
		if ( FXMessageBox::question(getMainWindow(),MBOX_YES_NO_CANCEL, "File New", "Save changes to Dialog?") == MBOX_CLICKED_YES )
		{
			doFileSave();
		}
	}

	// Write the registry
	reg().write();

	// Dump profile data
	#ifdef FX_ENABLE_PROFILING
	FXProfiler::speak();
	#endif

	delete this;

	// Exit the program
	::exit(0);

	return 1;
}

/* protected */
FXbool CAppDialog::getWidget(FXWindow* wnd, CConnector** ppConnector, FXint& nIndex)
{
	if ( !m_bRunning )
		return FALSE;

	if ( !wnd )
		return FALSE;

	FXint i;
 	for ( i = 0; i < m_Widgets.no(); i++ ) 
	{
		// Note: CDispenser::IsValidWidget may alter pWindow
		FXWindow* pWindow = wnd;
		FXWindow* pWidget = m_Widgets[i]->pWidget;
		FXbool    bValid  = m_Widgets[i]->pDispenser->IsValidWidget(&pWindow);

		if ( bValid && pWidget == pWindow )
		{
			*ppConnector = m_Widgets[i];
			nIndex = i;
			return TRUE; 
		} 
	} 

	return FALSE; 
}

FXbool CAppDialog::isValidWidget(FXWindow* wnd, CConnector** ppConnector) 
{
	FXint nIndex = 0;
	return getWidget(wnd, ppConnector, nIndex);
} 

FXbool CAppDialog::isValidWidget(FXWindow* wnd) 
{
	CConnector* pConnector = 0;
	FXint nIndex = 0;

	return getWidget(wnd, &pConnector, nIndex);
} 

FXint CAppDialog::getWidgetIndex(FXWindow* wnd, CConnector** ppConnector) 
{
	FXint nIndex = 0;
	getWidget(wnd, ppConnector, nIndex);
	return nIndex;
}

FXString getStringProperty(FXWindow* pWindow, IDispenser* pDispenser, const char* pszProperty)
{
	FXString str;
	IProperty*  pProperty = 0;

	FXint nCount = pDispenser->getPropertyCount();
	for (FXint i = 0; i < nCount; i++)
	{
		pProperty = pDispenser->getProperty(i);
		if ( strcmp(pProperty->name(), pszProperty) == 0 )
		{
			return pProperty->get(pWindow);
		}
	}

	return str;
}

void CAppDialog::addDispenser(IDispenser* pDispenser)
{
	DISPENSERDATA data;
	data.pDispenser = pDispenser;
	data.nCount = 0;
	m_Dispensers.append(data);
}

IDispenser* CAppDialog::findDispenser(FXString strName)
{
 	for ( FXint i = 0; i < m_Dispensers.no(); i++ ) 
	{
		if ( m_Dispensers[i].pDispenser->getWidgetName() == strName ) 
		{
			return m_Dispensers[i].pDispenser;
		} 
	} 

	return 0;
}

FXint CAppDialog::getDispenserCount(IDispenser* pDispenser)
{
 	for ( FXint i = 0; i < m_Dispensers.no(); i++ ) 
	{
		if ( m_Dispensers[i].pDispenser->getWidgetName() == pDispenser->getWidgetName() ) 
		{
			return ++m_Dispensers[i].nCount;
		} 
	} 

	return 1;
}

void CAppDialog::zeroDispenserCount(void)
{
 	for ( FXint i = 0; i < m_Dispensers.no(); i++ ) 
	{
		m_Dispensers[i].nCount = 0;
	} 
}

long CAppDialog::onFileNew(FXObject* o,FXSelector s,void* ptr)
{
	if ( m_bDirty )
	{
		switch ( FXMessageBox::question(getMainWindow(),MBOX_YES_NO_CANCEL, "File New", "Save changes to Dialog?") )
		{
		case MBOX_CLICKED_CANCEL:
			return 1;

		case MBOX_CLICKED_YES:
			if ( !doFileSave() )
			{
				return 1;
			}

		default:
			break;
		}
	}

	zeroDispenserCount();
	m_strFileName = "";
	clearWidgets();

	getTrackManager()->trackDialog();
	m_bDirty = FALSE;
	return 1;
}

long CAppDialog::onUpdateFileNew(FXObject* o,FXSelector,void* ptr)
{
//    FXuint msg = m_bDirty ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
//    o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CAppDialog::onFileOpen(FXObject* o,FXSelector,void* ptr)
{
	FXFileDialog dialog(getMainWindow(), "Open", 0, 50, 50);
	dialog.setPatternList(patterns);

	if ( !dialog.execute() )
	{
		return 1;
	}

	if ( m_bDirty )
	{
		switch ( FXMessageBox::question(getMainWindow(),MBOX_YES_NO_CANCEL, "File New", "Save changes to Dialog?") )
		{
		case MBOX_CLICKED_CANCEL:
			return 1;

		case MBOX_CLICKED_YES:
			if ( !doFileSave() )
			{
				return 1;
			}

		default:
			break;
		}
	}

	clearWidgets();

	FXString strFull = dialog.getFilename();
	FXString strFile = (strFull.findb('.') == -1) ? strFull : strFull.mid(0, strFull.findb('.'));

	FXFileStream DLGFile;
	FXString strfile = strFile + ".dlg";

	if ( DLGFile.open(strfile.text(), FXStreamLoad) )
	{
		FXString strWidgetCount;

		DLGFile >> strWidgetCount;

		FXint nWidgetCount = getIntFromString(strWidgetCount);

		for ( FXint w = 0; w < nWidgetCount; w++ ) 
		{
			CConnector* pConnector = 0;
			FXString    strWidgetName;
			FXString    strParentIndex;

			DLGFile >> strWidgetName >> strParentIndex;

			FXint nParentIndex = getIntFromString(strParentIndex);

			if ( w == 0 )
			{
				// It's the dialog itself
				pConnector = m_Widgets[1];
			}
			else
			{
				CConnector connector;

				connector.pDispenser    = findDispenser(strWidgetName);
				connector.pNames        = m_Widgets[0]->pNames;
				connector.pDescription  = m_Widgets[0]->pDescription;
				connector.pProperties   = m_Widgets[0]->pProperties;
				connector.pTrackManager = m_Widgets[0]->pTrackManager;

				connector.addWidget(m_Widgets[nParentIndex]->pWidget, TRUE);

				// CConnector::addWidget has cloned the connector above
				// and has cloned a new IDispenser*. The new IDispenser*
				// has also got an additional NameProperty
				// 
				// So lets get a pointer to the new CConnector and carry on...

				getWidgetIndex(connector.pWidget, &pConnector);
			}

			FXString strPropertyCount;
			
			DLGFile >> strPropertyCount;

			FXint nPropertyCount = getIntFromString(strPropertyCount);

			for (FXint i = 0; i < nPropertyCount; i++)
			{
				FXString strName;
				FXString strValue;
				DLGFile >> strName >> strValue;

				for (FXint j = 0; j < nPropertyCount; j++)
				{
					IProperty* pProperty = pConnector->pDispenser->getProperty(j);

					if ( pProperty->name() == strName )
					{
						pProperty->set(pConnector->pWidget, strValue);
					}
				}
			}
		}
	}

	// Select the dialog
	getTrackManager()->trackDialog();

	return 1;
}

FXbool CAppDialog::doFileSave(void)
{
	// m_Widgets[1] is the dialog frame
	IDispenser* pDlgDispenser = m_Widgets[1]->pDispenser;

	FXString strDlgName    = getStringProperty(m_Widgets[1]->pWidget, pDlgDispenser, "Name");
	FXString strDlgCaption = getStringProperty(m_Widgets[1]->pWidget, pDlgDispenser, "Caption");

	if ( m_strFileName.length() == 0 )
	{
		FXFileDialog dialog(getMainWindow(), "Save As", 0, 50, 50);

		dialog.setFilename(strDlgName + ".dlg");
		dialog.setPatternList(patterns);

		if ( !dialog.execute() )
		{
			return FALSE;
		}

		FXString strFull = dialog.getFilename();
		m_strFileName = (strFull.findb('.') == -1) ? strFull : strFull.mid(0, strFull.findb('.'));
	}

	m_bDirty = FALSE;

	CFile Hfile;
	FXString strHfile = m_strFileName + ".h";

	CFile CPPfile;
	FXString strCPPfile =  m_strFileName + ".cpp";

	FXbool bHExists   = Hfile.openRead(strHfile);
	FXbool bCPPExists = CPPfile.openRead(strCPPfile);

	Hfile.close();
	CPPfile.close();

	if ( bHExists || bCPPExists )
	{
		FXString strMessage;
		
		if ( bHExists )
		{
			strMessage += strHfile + "\n";
		}

		if ( bCPPExists )
		{
			strMessage += strCPPfile + "\n";
		}

		if ( bHExists && bCPPExists )
		{
			strMessage += "These files already exist.\nReplace existing files?";
		}
		else
		{
			strMessage += "This file already exists.\nReplace existing file?";
		}

		if ( MBOX_CLICKED_NO==FXMessageBox::question(getMainWindow(),MBOX_YES_NO,
			"File Save",strMessage.text()) )
		{
			return TRUE;
		}
	}

	if ( Hfile.openWrite(strHfile) )
	{
		Hfile << "class " << strDlgName << " : public FXDialogBox\n";
		Hfile << "{\n";
		Hfile << "\tFXDECLARE(" << strDlgName << ")\n";
		Hfile << "\n";
		Hfile << "// Services\n";
		Hfile << "public:\n";
		Hfile << "\t" << strDlgName << "(FXWindow* owner);\n";
		Hfile << "\n";
		Hfile << "private:\n";
		Hfile << "\t" << strDlgName << "() {}\n";
		Hfile << "\n";
		Hfile << "// Attributes\n";
		Hfile << "protected:\n";

 		// Start at 3 as we don't want to add m_pAppPane and m_pPane!
		for ( FXint i = 3; i < m_Widgets.no(); i++ ) 
		{
			IDispenser* pChildDispenser = m_Widgets[i]->pDispenser;
			FXWindow*   pParent         = m_Widgets[i]->pWidget->getParent();

			FXString strWidgetName = pChildDispenser->getWidgetName();

			Hfile << "\t" << strWidgetName << "* m_p" << getStringProperty(m_Widgets[i]->pWidget, pChildDispenser, "Name") << ";\n";

			CConnector* pParentConnector = 0;

			FXString strParent = "ERROR";

			// Widget 2 is the dialog pane
			if ( pParent == m_Widgets[2]->pWidget )
			{
				strParent = "this";
			}
			else if ( isValidWidget(pParent, &pParentConnector) )
			{
				strParent = "m_p" + getStringProperty(m_Widgets[i]->pWidget, pParentConnector->pDispenser, "Name");
			}
			
			FXString strName = "m_p" + getStringProperty(m_Widgets[i]->pWidget, pChildDispenser, "Name");
			
			FXString strSuffix;
			if ( pParentConnector )
				strSuffix = pParentConnector->pDispenser->getChildSuffix();

			FXString strFullParent = strParent + strSuffix;
			pChildDispenser->getDefinitionCode(Hfile, m_Widgets[i]->pWidget, strName, strFullParent);
		} 

		Hfile << "};\n";
	}

	if ( CPPfile.openWrite(strCPPfile) )
	{
		FXString strFileName = (m_strFileName.findb('/') == -1) 
			? m_strFileName.mid(m_strFileName.findb('\\')+1, m_strFileName.length() - m_strFileName.findb('\\')) 
			: m_strFileName.mid(m_strFileName.findb('/')+1 , m_strFileName.length() - m_strFileName.findb('/'));

		CPPfile << "#include <fx.h>\n";
		CPPfile << "\n";
		CPPfile << "#include " << char(34) << strFileName << ".h" << char(34) << "\n";

        // allenb - It didn't like the passing by ref of the obj directly
		FXString strEmptyString("");
		pDlgDispenser->getImplementationCode(CPPfile, m_Widgets[1]->pWidget, strDlgName, strEmptyString);

		CPPfile << "{\n";

		for ( FXint i = 3; i < m_Widgets.no(); i++ ) 
		{
			// We don't want to add the m_pAppPane or m_pPane!
			//if ( i == 0 || i == 2 )
			//	continue;

			IDispenser* pChildDispenser = m_Widgets[i]->pDispenser;
			FXWindow*   pParent         = m_Widgets[i]->pWidget->getParent();

			CConnector* pParentConnector = 0;

			FXString strParent = "ERROR";

			// Widget 2 is the dialog pane
			if ( pParent == m_Widgets[2]->pWidget )
			{
				strParent = "this";
			}
			else if ( isValidWidget(pParent, &pParentConnector) )
			{
				strParent = "m_p" + getStringProperty(m_Widgets[i]->pWidget, pParentConnector->pDispenser, "Name");
			}
			
			FXString strName = "m_p" + getStringProperty(m_Widgets[i]->pWidget, pChildDispenser, "Name");
			
			FXString strSuffix;
			if ( pParentConnector )
				strSuffix = pParentConnector->pDispenser->getChildSuffix();

			FXString strFullParent = strParent + strSuffix;
			pChildDispenser->getImplementationCode(CPPfile, m_Widgets[i]->pWidget, strName, strFullParent);
		} 

		CPPfile << "}\n";
	}

	FXFileStream DLGFile;
	FXString strDLGfile =  m_strFileName + ".dlg";
	if ( DLGFile.open(strDLGfile.text(), FXStreamSave) )
	{
		FXint nWidgets = m_Widgets.no() - 3 + 1;

		DLGFile << getStringFromInt(nWidgets);

		for ( FXint i = 0; i < m_Widgets.no(); i++ ) 
		{
			// We don't want to add the m_pAppPane or m_pPane!
			if ( i == 0 || i == 2 )
				continue;

			CConnector* pConnector    = 0;
			IDispenser* pDispenser    = m_Widgets[i]->pDispenser;
			FXString    strWidgetName = pDispenser->getWidgetName();
			FXWindow*   pWidget       = m_Widgets[i]->pWidget;
			FXint       nParent       = getWidgetIndex(pWidget->getParent(), &pConnector); 
 
			DLGFile << strWidgetName << getStringFromInt(nParent);

			FXint nCount = pDispenser->getPropertyCount();

			DLGFile << getStringFromInt(nCount);

			for (FXint i = 0; i < nCount; i++)
			{
				IProperty* pProperty     = pDispenser->getProperty(i);
				FXString strName         = pProperty->name();
				FXString strValue        = pProperty->get(pWidget);

				DLGFile << strName << strValue;
			}
		}
	}

	return TRUE;
}

long CAppDialog::onFileSave(FXObject* o,FXSelector,void* ptr)
{
	doFileSave();

	return 1;
}

long CAppDialog::onFileSaveAs(FXObject* o,FXSelector s,void* ptr)
{
	m_strFileName = "";

	doFileSave();

	return 1;
}

///////////////////////////////////////////////////////////////////////////////
//

