/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserDial.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>
#include <CPropertyRange.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyDialStyle : public CProperty
{
public:
	CPropertyDialStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	virtual void set(FXWindow* pWindow, FXString strValue);
	virtual FXString get(FXWindow* pWindow);
	virtual void validate(IConnector* pConnector, FXString& strNewValue);
	virtual void validate(IConnector* pConnector) {}

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

///////////////////////////////////////////////////////////////////////////////
//

// From FXDial.h
template <class T>
OPTION CPropertyDialStyle<T>::Style[] = 
{
	{"DIAL_VERTICAL", DIAL_VERTICAL, "Vertically oriented"},
	{"DIAL_HORIZONTAL", DIAL_HORIZONTAL, "Horizontal oriented"},
	{"DIAL_CYCLIC",  DIAL_CYCLIC,  "Value wraps around"},
	{"DIAL_HAS_NOTCH",  DIAL_HAS_NOTCH,  "Dial has a Center Notch"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyDialStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyDialStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyDialStyle<T>::description(void)
{
	return "The dial style for this object";
}

template <class T>
const char* CPropertyDialStyle<T>::tooltip(void)
{
	return "getDialStyle()\nsetDialStyle()";
}

template <class T>
void CPropertyDialStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setDialStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyDialStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getDialStyle() );
}

template <class T>
void CPropertyDialStyle<T>::validate(IConnector* pConnector, FXString& strNewValue)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nOldValue = p->getDialStyle();
	FXint nNewValue = getIntFromString(strNewValue);

	toggleAttribute(nOldValue, nNewValue, DIAL_VERTICAL, DIAL_HORIZONTAL);

	strNewValue = getStringFromInt(nNewValue);
}

template <class T>
OPTION* CPropertyDialStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyDialStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getDialStyle() == 0 )
		return;
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXColor C>
class CPropertyNotchColor : public CProperty
{
public:
	CPropertyNotchColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertyNotchColor<T, C>::name(void)
{
	return "NotchColor";
}

template <class T, FXColor C>
ePropertyType CPropertyNotchColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertyNotchColor<T, C>::description(void)
{
	return "The notch color of this object";
}

template <class T, FXColor C>
const char* CPropertyNotchColor<T, C>::tooltip(void)
{
	return "getNotchColor()\nsetNotchColor()";
}

template <class T, FXColor C>
void CPropertyNotchColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setNotchColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertyNotchColor<T, C>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getNotchColor() );
}

template <class T, FXColor C>
FXString CPropertyNotchColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getNotchColor() != C )
	{
		FXString str = getFXRGBString(p->getNotchColor());
		return "\t" + widget + "->setNotchColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Dial[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x04,0x04,0x04,0x0c,0x0c,
  0x0c,0x11,0x11,0x11,0x16,0x16,0x16,0x1c,0x1c,0x1c,0x22,0x22,0x22,0x55,0x55,0x55,
  0x4d,0x4d,0x4d,0x42,0x42,0x42,0x39,0x39,0x39,0xff,0x7c,0x80,0xff,0x50,0x50,0xd6,
  0x00,0x93,0xcc,0xec,0xff,0xef,0xd6,0xc6,0xe7,0xe7,0xd6,0xad,0xa9,0x90,0x33,0x00,
  0x00,0x66,0x00,0x00,0x99,0x00,0x00,0x00,0x33,0x00,0x33,0x33,0x00,0x66,0x33,0x00,
  0x99,0x33,0x00,0xcc,0x33,0x00,0xff,0x33,0x00,0x00,0x66,0x00,0x33,0x66,0x00,0x66,
  0x66,0x00,0x99,0x66,0x00,0xcc,0x66,0x00,0xff,0x66,0x00,0x00,0x99,0x00,0x33,0x99,
  0x00,0x66,0x99,0x00,0x99,0x99,0x00,0xcc,0x99,0x00,0xff,0x99,0x00,0x00,0xcc,0x00,
  0x33,0xcc,0x00,0x66,0xcc,0x00,0x99,0xcc,0x00,0xcc,0xcc,0x00,0xff,0xcc,0x00,0x66,
  0xff,0x00,0x99,0xff,0x00,0xcc,0xff,0x00,0x00,0x00,0x33,0x33,0x00,0x33,0x66,0x00,
  0x33,0x99,0x00,0x33,0xcc,0x00,0x33,0xff,0x00,0x33,0x00,0x33,0x33,0x33,0x33,0x33,
  0x66,0x33,0x33,0x99,0x33,0x33,0xcc,0x33,0x33,0xff,0x33,0x33,0x00,0x66,0x33,0x33,
  0x66,0x33,0x66,0x66,0x33,0x99,0x66,0x33,0xcc,0x66,0x33,0xff,0x66,0x33,0x00,0x99,
  0x33,0x33,0x99,0x33,0x66,0x99,0x33,0x99,0x99,0x33,0xcc,0x99,0x33,0xff,0x99,0x33,
  0x00,0xcc,0x33,0x33,0xcc,0x33,0x66,0xcc,0x33,0x99,0xcc,0x33,0xcc,0xcc,0x33,0xff,
  0xcc,0x33,0x33,0xff,0x33,0x66,0xff,0x33,0x99,0xff,0x33,0xcc,0xff,0x33,0xff,0xff,
  0x33,0x00,0x00,0x66,0x33,0x00,0x66,0x66,0x00,0x66,0x99,0x00,0x66,0xcc,0x00,0x66,
  0xff,0x00,0x66,0x00,0x33,0x66,0x33,0x33,0x66,0x66,0x33,0x66,0x99,0x33,0x66,0xcc,
  0x33,0x66,0xff,0x33,0x66,0x00,0x66,0x66,0x33,0x66,0x66,0x66,0x66,0x66,0x99,0x66,
  0x66,0xcc,0x66,0x66,0x00,0x99,0x66,0x33,0x99,0x66,0x66,0x99,0x66,0x99,0x99,0x66,
  0xcc,0x99,0x66,0xff,0x99,0x66,0x00,0xcc,0x66,0x33,0xcc,0x66,0x99,0xcc,0x66,0xcc,
  0xcc,0x66,0xff,0xcc,0x66,0x00,0xff,0x66,0x33,0xff,0x66,0x99,0xff,0x66,0xcc,0xff,
  0x66,0xff,0x00,0xcc,0xcc,0x00,0xff,0x00,0x99,0x99,0x99,0x33,0x99,0x99,0x00,0x99,
  0xcc,0x00,0x99,0x00,0x00,0x99,0x33,0x33,0x99,0x66,0x00,0x99,0xcc,0x33,0x99,0xff,
  0x00,0x99,0x00,0x66,0x99,0x33,0x66,0x99,0x66,0x33,0x99,0x99,0x66,0x99,0xcc,0x66,
  0x99,0xff,0x33,0x99,0x33,0x99,0x99,0x66,0x99,0x99,0x99,0x99,0x99,0xcc,0x99,0x99,
  0xff,0x99,0x99,0x00,0xcc,0x99,0x33,0xcc,0x99,0x66,0xcc,0x66,0x99,0xcc,0x99,0xcc,
  0xcc,0x99,0xff,0xcc,0x99,0x00,0xff,0x99,0x33,0xff,0x99,0x66,0xcc,0x99,0x99,0xff,
  0x99,0xcc,0xff,0x99,0xff,0xff,0x99,0x00,0x00,0xcc,0x33,0x00,0x99,0x66,0x00,0xcc,
  0x99,0x00,0xcc,0xcc,0x00,0xcc,0x00,0x33,0x99,0x33,0x33,0xcc,0x66,0x33,0xcc,0x99,
  0x33,0xcc,0xcc,0x33,0xcc,0xff,0x33,0xcc,0x00,0x66,0xcc,0x33,0x66,0xcc,0x66,0x66,
  0x99,0x99,0x66,0xcc,0xcc,0x66,0xcc,0xff,0x66,0x99,0x00,0x99,0xcc,0x33,0x99,0xcc,
  0x66,0x99,0xcc,0x99,0x99,0xcc,0xcc,0x99,0xcc,0xff,0x99,0xcc,0x00,0xcc,0xcc,0x33,
  0xcc,0xcc,0x66,0xcc,0xcc,0x99,0xcc,0xcc,0xcc,0xcc,0xcc,0xff,0xcc,0xcc,0x00,0xff,
  0xcc,0x33,0xff,0xcc,0x66,0xff,0x99,0x99,0xff,0xcc,0xcc,0xff,0xcc,0xff,0xff,0xcc,
  0x33,0x00,0xcc,0x66,0x00,0xff,0x99,0x00,0xff,0x00,0x33,0xcc,0x33,0x33,0xff,0x66,
  0x33,0xff,0x99,0x33,0xff,0xcc,0x33,0xff,0xff,0x33,0xff,0x00,0x66,0xff,0x33,0x66,
  0xff,0x66,0x66,0xcc,0x99,0x66,0xff,0xcc,0x66,0xff,0xff,0x66,0xcc,0x00,0x99,0xff,
  0x33,0x99,0xff,0x66,0x99,0xff,0x99,0x99,0xff,0xcc,0x99,0xff,0xff,0x99,0xff,0x00,
  0xcc,0xff,0x33,0xcc,0xff,0x66,0xcc,0xff,0x08,0x08,0x08,0x99,0xcc,0xff,0xcc,0xcc,
  0xff,0xff,0xcc,0xff,0x33,0xff,0xff,0x66,0xff,0xcc,0x29,0x29,0x29,0x99,0xff,0xff,
  0xcc,0xff,0xff,0xff,0x66,0x66,0x66,0xff,0x66,0xff,0xff,0x66,0x66,0x66,0xff,0xff,
  0x66,0xff,0x66,0xff,0xff,0xa5,0x00,0x21,0x5f,0x5f,0x5f,0x77,0x77,0x77,0x86,0x86,
  0x86,0x96,0x96,0x96,0xcb,0xcb,0xcb,0xb2,0xb2,0xb2,0xd7,0xd7,0xd7,0xdd,0xdd,0xdd,
  0xe3,0xe3,0xe3,0xea,0xea,0xea,0xf1,0xf1,0xf1,0xf8,0xf8,0xf8,0xcc,0x00,0x00,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xf1,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x1c,0xb8,0xae,0xa1,0xc3,0x85,0x10,0x23,0x4a,0x44,0xe8,
  0xf0,0xe1,0xc4,0x8b,0x18,0x13,0x56,0x6c,0x98,0xb1,0xa3,0x47,0x7c,0x1b,0x43,0x8a,
  0x1c,0x69,0x71,0x22,0xc9,0x93,0x27,0x01,0xa8,0x5c,0xc9,0xb2,0xa5,0xcb,0x97,0x30,
  0x63,0xc2,0xfc,0x97,0xf2,0x80,0xcd,0x9b,0x38,0x73,0xea,0xdc,0xc9,0xb3,0xe7,0x4e,
  0x9a,0x24,0x01,0x1c,0xf8,0x48,0xf4,0xe0,0x01,0xa0,0x23,0x85,0x0a,0x94,0xc9,0xb4,
  0xa9,0x53,0x81,0x47,0x6b,0x16,0x9d,0x3a,0x30,0x6a,0xd0,0xa1,0xf8,0x9c,0x6a,0xdd,
  0xca,0x12,0x2a,0x52,0x91,0x4a,0xf1,0xfd,0x1b,0x4b,0xb6,0xac,0xd9,0xb3,0x68,0xd3,
  0xaa,0xfd,0xe7,0x55,0x6a,0x56,0xae,0x70,0x9b,0xb6,0xbd,0x0a,0xd5,0xa7,0xdd,0xbb,
  0x77,0xe7,0x26,0xc5,0xba,0xb6,0xaf,0xdf,0xbf,0x7a,0xc1,0x62,0x8d,0x4b,0x38,0x66,
  0xe0,0x90,0x61,0xf1,0x2a,0x5e,0x8c,0xf3,0xf0,0xc6,0xb0,0x7f,0x23,0x4b,0x2e,0xeb,
  0xb8,0xa2,0xd2,0x03,0x85,0x33,0xb7,0x1c,0x6a,0x75,0x2f,0x3e,0xc6,0xa0,0x17,0x7f,
  0xfe,0x8a,0x98,0x73,0xe8,0xd3,0x3e,0x47,0x4b,0x3d,0x3a,0xb9,0xb5,0x5f,0xce,0xa4,
  0x1f,0x73,0xd6,0x4c,0xfb,0x72,0x6c,0xcb,0xa6,0x51,0xeb,0xce,0xa9,0x7a,0x64,0xd6,
  0xdc,0xbb,0x83,0xf7,0x16,0xf9,0x7b,0xb4,0xeb,0xe3,0x69,0x61,0x93,0x2c,0xce,0x1a,
  0xb9,0x73,0xb2,0xca,0xe9,0x62,0xae,0x9d,0x39,0xba,0xe7,0xe0,0xd8,0xad,0x0b,0x16,
  0x9b,0x5d,0x37,0xdb,0xce,0xdb,0x9f,0xb6,0x8b,0x27,0x3b,0x3c,0xfc,0x78,0xf1,0xe5,
  0x4b,0x8b,0xa5,0x9e,0xf9,0xfb,0x6d,0x87,0x4a,0xff,0x75,0x47,0xed,0x7e,0xf5,0x7c,
  0xd4,0xe9,0x65,0x1b,0x3f,0x7f,0x5c,0xbb,0xfa,0xe6,0xfc,0xb5,0xe6,0x9f,0x7e,0xd3,
  0xb1,0x17,0xd7,0x80,0xb8,0x7d,0x76,0xdf,0x69,0xf9,0x25,0xb8,0x20,0x83,0xe0,0xfd,
  0x17,0x60,0x7f,0x0d,0xc2,0x37,0x9b,0x81,0x07,0x56,0xd8,0xd0,0x65,0x0f,0x82,0xa6,
  0xe1,0x3a,0x1c,0x76,0x28,0x5a,0x84,0x04,0x4e,0xe8,0x1a,0x82,0x16,0x2e,0x85,0x21,
  0x5c,0x95,0xa5,0xa8,0xa0,0x88,0x79,0x7d,0x08,0x99,0x89,0x93,0xb5,0xb8,0xe1,0x60,
  0x2b,0x72,0x65,0x23,0x88,0x58,0xc1,0x88,0xd7,0x8e,0x33,0xd2,0x18,0x19,0x90,0x38,
  0xe6,0xa8,0x15,0x91,0x54,0x51,0x45,0x62,0x82,0x6f,0x19,0xf9,0x94,0x8c,0x58,0x25,
  0x49,0xd4,0x92,0x2e,0x3a,0x79,0x24,0x94,0x3e,0x82,0xf6,0xde,0x8d,0x59,0x32,0xb6,
  0x25,0x88,0x42,0x3e,0x87,0xd2,0x98,0x64,0x72,0x04,0x51,0x99,0x68,0xa6,0xa9,0xe6,
  0x9a,0x6c,0xb6,0xe9,0xe6,0x9b,0x70,0xc6,0x29,0xe7,0x9c,0x65,0x06,0x04,0x00,0x3b
  };

CDispenserDial::CDispenserDial()
{
	m_Properties.append(new CPropertyLayout<FXDial>(this));
	m_Properties.append(new CPropertyFrame<FXDial>(this));
	m_Properties.append(new CPropertyDialStyle<FXDial>(this));
	m_Properties.append(new CPropertyBackColor<FXDial, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyNotchColor<FXDial, FXRGB(255, 128, 0)>(this));
//	m_Properties.append(new CPropertyRange<FXDial>(this));
	m_Properties.append(new CPropertyPadTop<FXDial>(this));
	m_Properties.append(new CPropertyPadLeft<FXDial>(this));
	m_Properties.append(new CPropertyPadBottom<FXDial>(this));
	m_Properties.append(new CPropertyPadRight<FXDial>(this));
	m_Properties.append(new CPropertyLeft<FXDial>(this));
	m_Properties.append(new CPropertyTop<FXDial>(this));
	m_Properties.append(new CPropertyWidth<FXDial>(this));
	m_Properties.append(new CPropertyHeight<FXDial>(this));
}

const char* CDispenserDial::getWidgetName(void)
{
	return "FXDial";
}

const void* CDispenserDial::getWidgetIcon(void)
{
	return Dial;
}

const char* CDispenserDial::getWidgetCategory(void)
{
	return "Extended";
}

const int   CDispenserDial::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserDial::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserDial::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserDial::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserDial::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserDial::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserDial::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserDial::getClassInfo(void)
{
	return &FXDial::metaClass;
}

FXbool CDispenserDial::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserDial::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserDial::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserDial::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXDial::metaClass);
}

IDispenser* CDispenserDial::clone(void)
{
	return new CDispenserDial;
}

FXWindow* CDispenserDial::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXDial(p, tgt, sel,
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserDial::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXDial(" << parent << ", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream  << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
