/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserProgressBar.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyProgressStyle : public CProperty
{
public:
	CPropertyProgressStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);

	virtual void set(FXWindow* pWindow, FXString strValue);
	virtual FXString get(FXWindow* pWindow);
	virtual void validate(IConnector* pConnector, FXString& strNewValue);
	virtual void validate(IConnector* pConnector) {}

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

template <class T>
OPTION CPropertyProgressStyle<T>::Style[] = 
{
	{"PROGRESSBAR_PERCENTAGE", PROGRESSBAR_PERCENTAGE ,"Show percentage done"},
	{"PROGRESSBAR_VERTICAL"  , PROGRESSBAR_VERTICAL   ,"Vertical display"},
	{"PROGRESSBAR_HORIZONTAL", PROGRESSBAR_HORIZONTAL ,"Horizontal display"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyProgressStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyProgressStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyProgressStyle<T>::description(void)
{
	return "The progress style of this object";
}

template <class T>
void CPropertyProgressStyle<T>::set(FXWindow* pWindow, const FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setBarStyle(getIntFromString(strValue));
}

template <class T>
FXString CPropertyProgressStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt(p->getBarStyle());
}

template <class T>
void CPropertyProgressStyle<T>::validate(IConnector* pConnector, FXString& strNewValue)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nOldValue = p->getBarStyle();
	FXint nNewValue = getIntFromString(strNewValue);

	toggleAttribute(nOldValue, nNewValue, PROGRESSBAR_VERTICAL, PROGRESSBAR_HORIZONTAL);

	strNewValue = getStringFromInt(nNewValue);
}

template <class T>
OPTION* CPropertyProgressStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyProgressStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getBarStyle() == PROGRESSBAR_HORIZONTAL )
		return;

	CProperty::getOptions(pWindow, strOptions);
}


///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXColor C>
class CPropertyBarColor : public CProperty
{
public:
	CPropertyBarColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertyBarColor<T, C>::name(void)
{
	return "BarColor";
}

template <class T, FXColor C>
ePropertyType CPropertyBarColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertyBarColor<T, C>::description(void)
{
	return "The bar color of this object";
}

template <class T, FXColor C>
const char* CPropertyBarColor<T, C>::tooltip(void)
{
	return "getBarColor()\nsetBarColor()";
}

template <class T, FXColor C>
void CPropertyBarColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setBarColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertyBarColor<T, C>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getBarColor() );
}

template <class T, FXColor C>
FXString CPropertyBarColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getBarColor() != C )
	{
		FXString str = getFXRGBString(p->getBarColor());
		return "\t" + widget + "->setBarColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXColor C>
class CPropertyBarBackColor : public CProperty
{
public:
	CPropertyBarBackColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertyBarBackColor<T, C>::name(void)
{
	return "BarBackColor";
}

template <class T, FXColor C>
ePropertyType CPropertyBarBackColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertyBarBackColor<T, C>::description(void)
{
	return "The background bar color of this object";
}

template <class T, FXColor C>
const char* CPropertyBarBackColor<T, C>::tooltip(void)
{
	return "getBarBGColor()\nsetBarBGColor()";
}

template <class T, FXColor C>
void CPropertyBarBackColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setBarBGColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertyBarBackColor<T, C>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getBarBGColor() );
}

template <class T, FXColor C>
FXString CPropertyBarBackColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getBarBGColor() != C )
	{
		FXString str = getFXRGBString(p->getBarBGColor());
		return "\t" + widget + "->setBarBGColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXColor C>
class CPropertyTextAltColor : public CProperty
{
public:
	CPropertyTextAltColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertyTextAltColor<T, C>::name(void)
{
	return "TextAltColor";
}

template <class T, FXColor C>
ePropertyType CPropertyTextAltColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertyTextAltColor<T, C>::description(void)
{
	return "The alternate text color\nof this object";
}

template <class T, FXColor C>
const char* CPropertyTextAltColor<T, C>::tooltip(void)
{
	return "getTextAltColor()\nsetTextAltColor()";
}

template <class T, FXColor C>
void CPropertyTextAltColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setTextAltColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertyTextAltColor<T, C>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getTextAltColor() );
}

template <class T, FXColor C>
FXString CPropertyTextAltColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getTextAltColor() != C )
	{
		FXString str = getFXRGBString(p->getTextAltColor());
		return "\t" + widget + "->setTextAltColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Progress[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x00,0x00,0x55,0x00,0x00,0x7f,0x00,0x00,0xaa,0x00,0x00,0xd4,0x00,0x2a,0x00,
  0x00,0x2a,0x2a,0x00,0x2a,0x55,0x00,0x2a,0x7f,0x00,0x2a,0xaa,0x00,0x2a,0xd4,0x00,
  0x55,0x00,0x00,0x55,0x2a,0x00,0x55,0x55,0x00,0x55,0x7f,0x00,0x55,0xaa,0x00,0x55,
  0xd4,0x00,0x7f,0x00,0x00,0x7f,0x2a,0x00,0x7f,0x55,0x00,0x7f,0x7f,0x00,0x7f,0xaa,
  0x00,0x7f,0xd4,0x00,0xaa,0x00,0x00,0xaa,0x2a,0x00,0xaa,0x55,0x00,0xaa,0x7f,0x00,
  0xaa,0xaa,0x00,0xaa,0xd4,0x00,0xd4,0x00,0x00,0xd4,0x2a,0x00,0xd4,0x55,0x00,0xd4,
  0x7f,0x00,0xd4,0xaa,0x00,0xd4,0xd4,0x2a,0x00,0x00,0x2a,0x00,0x2a,0x2a,0x00,0x55,
  0x2a,0x00,0x7f,0x2a,0x00,0xaa,0x2a,0x00,0xd4,0x2a,0x2a,0x00,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x55,0x2a,0x2a,0x7f,0x2a,0x2a,0xaa,0x2a,0x2a,0xd4,0x2a,0x55,0x00,0x2a,0x55,
  0x2a,0x2a,0x55,0x55,0x2a,0x55,0x7f,0x2a,0x55,0xaa,0x2a,0x55,0xd4,0x2a,0x7f,0x00,
  0x2a,0x7f,0x2a,0x2a,0x7f,0x55,0x2a,0x7f,0x7f,0x2a,0x7f,0xaa,0x2a,0x7f,0xd4,0x2a,
  0xaa,0x00,0x2a,0xaa,0x2a,0x2a,0xaa,0x55,0x2a,0xaa,0x7f,0x2a,0xaa,0xaa,0x2a,0xaa,
  0xd4,0x2a,0xd4,0x00,0x2a,0xd4,0x2a,0x2a,0xd4,0x55,0x2a,0xd4,0x7f,0x2a,0xd4,0xaa,
  0x2a,0xd4,0xd4,0x55,0x00,0x00,0x55,0x00,0x2a,0x55,0x00,0x55,0x55,0x00,0x7f,0x55,
  0x00,0xaa,0x55,0x00,0xd4,0x55,0x2a,0x00,0x55,0x2a,0x2a,0x55,0x2a,0x55,0x55,0x2a,
  0x7f,0x55,0x2a,0xaa,0x55,0x2a,0xd4,0x55,0x55,0x00,0x55,0x55,0x2a,0x55,0x55,0x55,
  0x55,0x55,0x7f,0x55,0x55,0xaa,0x55,0x55,0xd4,0x55,0x7f,0x00,0x55,0x7f,0x2a,0x55,
  0x7f,0x55,0x55,0x7f,0x7f,0x55,0x7f,0xaa,0x55,0x7f,0xd4,0x55,0xaa,0x00,0x55,0xaa,
  0x2a,0x55,0xaa,0x55,0x55,0xaa,0x7f,0x55,0xaa,0xaa,0x55,0xaa,0xd4,0x55,0xd4,0x00,
  0x55,0xd4,0x2a,0x55,0xd4,0x55,0x55,0xd4,0x7f,0x55,0xd4,0xaa,0x55,0xd4,0xd4,0x7f,
  0x00,0x00,0x7f,0x00,0x2a,0x7f,0x00,0x55,0x7f,0x00,0x7f,0x7f,0x00,0xaa,0x7f,0x00,
  0xd4,0x7f,0x2a,0x00,0x7f,0x2a,0x2a,0x7f,0x2a,0x55,0x7f,0x2a,0x7f,0x7f,0x2a,0xaa,
  0x7f,0x2a,0xd4,0x7f,0x55,0x00,0x7f,0x55,0x2a,0x7f,0x55,0x55,0x7f,0x55,0x7f,0x7f,
  0x55,0xaa,0x7f,0x55,0xd4,0x7f,0x7f,0x00,0x7f,0x7f,0x2a,0x7f,0x7f,0x55,0x7f,0x7f,
  0x7f,0x7f,0x7f,0xaa,0x7f,0x7f,0xd4,0x7f,0xaa,0x00,0x7f,0xaa,0x2a,0x7f,0xaa,0x55,
  0x7f,0xaa,0x7f,0x7f,0xaa,0xaa,0x7f,0xaa,0xd4,0x7f,0xd4,0x00,0x7f,0xd4,0x2a,0x7f,
  0xd4,0x55,0x7f,0xd4,0x7f,0x7f,0xd4,0xaa,0x7f,0xd4,0xd4,0xaa,0x00,0x00,0xaa,0x00,
  0x2a,0xaa,0x00,0x55,0xaa,0x00,0x7f,0xaa,0x00,0xaa,0xaa,0x00,0xd4,0xaa,0x2a,0x00,
  0xaa,0x2a,0x2a,0xaa,0x2a,0x55,0xaa,0x2a,0x7f,0xaa,0x2a,0xaa,0xaa,0x2a,0xd4,0xaa,
  0x55,0x00,0xaa,0x55,0x2a,0xaa,0x55,0x55,0xaa,0x55,0x7f,0xaa,0x55,0xaa,0xaa,0x55,
  0xd4,0xaa,0x7f,0x00,0xaa,0x7f,0x2a,0xaa,0x7f,0x55,0xaa,0x7f,0x7f,0xaa,0x7f,0xaa,
  0xaa,0x7f,0xd4,0xaa,0xaa,0x00,0xaa,0xaa,0x2a,0xaa,0xaa,0x55,0xaa,0xaa,0x7f,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xd4,0xaa,0xd4,0x00,0xaa,0xd4,0x2a,0xaa,0xd4,0x55,0xaa,0xd4,
  0x7f,0xaa,0xd4,0xaa,0xaa,0xd4,0xd4,0xd4,0x00,0x00,0xd4,0x00,0x2a,0xd4,0x00,0x55,
  0xd4,0x00,0x7f,0xd4,0x00,0xaa,0xd4,0x00,0xd4,0xd4,0x2a,0x00,0xd4,0x2a,0x2a,0xd4,
  0x2a,0x55,0xd4,0x2a,0x7f,0xd4,0x2a,0xaa,0xd4,0x2a,0xd4,0xd4,0x55,0x00,0xd4,0x55,
  0x2a,0xd4,0x55,0x55,0xd4,0x55,0x7f,0xd4,0x55,0xaa,0xd4,0x55,0xd4,0xd4,0x7f,0x00,
  0xd4,0x7f,0x2a,0xd4,0x7f,0x55,0xd4,0x7f,0x7f,0xd4,0x7f,0xaa,0xd4,0x7f,0xd4,0xd4,
  0xaa,0x00,0xd4,0xaa,0x2a,0xd4,0xaa,0x55,0xd4,0xaa,0x7f,0xd4,0xaa,0xaa,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0xd4,0xd4,0x2a,0xd4,0xd4,0x55,0xd4,0xd4,0x7f,0xd4,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0x00,0x00,0x0c,0x0c,0x0c,0x19,0x19,0x19,0x26,0x26,0x26,0x33,
  0x33,0x33,0x3f,0x3f,0x3f,0x4c,0x4c,0x4c,0x59,0x59,0x59,0x66,0x66,0x66,0x72,0x72,
  0x72,0x7f,0x7f,0x7f,0x8c,0x8c,0x8c,0x99,0x99,0x99,0xa5,0xa5,0xa5,0xb2,0xb2,0xb2,
  0xbf,0xbf,0xbf,0xcc,0xcc,0xcc,0xd8,0xd8,0xd8,0xe5,0xe5,0xe5,0xf2,0xf2,0xf2,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xf1,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0x6a,0xdc,0xc8,0xb1,0xa3,0xc7,0x8f,0x20,0x43,0x8a,
  0x1c,0x49,0xb2,0xa4,0xc9,0x93,0x28,0x53,0xaa,0x5c,0xc9,0xb2,0xa5,0xcb,0x97,0x30,
  0x5b,0x1e,0x98,0x49,0xb3,0xa6,0xcd,0x9b,0x38,0x73,0xea,0xd4,0xa9,0x70,0xa7,0xcf,
  0x9f,0x40,0x73,0xf6,0x3c,0x10,0xf3,0xdf,0xcc,0xa1,0xf8,0x00,0x28,0x5d,0xca,0xb4,
  0xa9,0xd3,0xa7,0x50,0xa3,0x2a,0x0d,0x67,0x94,0x68,0xc2,0xa3,0x00,0xf8,0x69,0xdd,
  0xca,0x55,0xeb,0xbf,0xaf,0x60,0xc3,0x8a,0x1d,0x4b,0x76,0x2c,0xd5,0xa3,0x57,0x89,
  0x66,0xed,0xca,0xb6,0xac,0xdb,0xb7,0x64,0xcf,0x5a,0x45,0x88,0x95,0x6d,0x5b,0xb8,
  0x78,0xdf,0xca,0x45,0xba,0xd6,0xee,0xd6,0xbc,0x80,0xe3,0x56,0xe5,0xeb,0x97,0x6b,
  0xe0,0xc3,0x60,0xf7,0xa6,0x4d,0x5a,0xf8,0xef,0x3f,0xc7,0x8f,0xbd,0x7e,0x95,0xfc,
  0x18,0xb0,0x62,0xba,0x6a,0x1b,0x7b,0xe5,0x07,0x96,0x33,0xe7,0xc9,0x95,0x43,0x7f,
  0xc6,0x7b,0xf9,0x60,0x5d,0xcd,0x62,0x47,0x83,0xfe,0xec,0x39,0x70,0x69,0x83,0xa7,
  0x1b,0x77,0x56,0x4d,0x59,0xb2,0x6a,0xb8,0xaf,0x0b,0xc6,0x2e,0x9c,0xba,0x77,0x58,
  0xd6,0xb7,0x05,0xa3,0xc5,0xcc,0x18,0xf5,0x6f,0xdf,0x9d,0x45,0xe3,0x1e,0xbc,0xb8,
  0x2f,0xef,0xd1,0xad,0x41,0x27,0x57,0xae,0x97,0x39,0x71,0xe7,0x7e,0x27,0xd7,0xa6,
  0x2c,0x3d,0x32,0x69,0xeb,0xa6,0x33,0x65,0x1b,0x47,0xec,0x1a,0x3c,0x6c,0xf1,0xb2,
  0xc9,0x97,0x1f,0x1e,0xbe,0x78,0x7a,0xf5,0x79,0x73,0x13,0xdc,0x9d,0x1d,0x7e,0x7c,
  0xf3,0xba,0xd1,0xf3,0xb6,0xff,0x9d,0xfd,0x79,0xf7,0xfb,0xf1,0x57,0x9d,0x7f,0xf9,
  0xe1,0x13,0xce,0x81,0x08,0x26,0xa8,0xe0,0x82,0x0c,0x36,0xe8,0x20,0x82,0xf8,0xcd,
  0x47,0x94,0x80,0x14,0x7e,0x45,0xa0,0x84,0x41,0x65,0xa8,0x21,0x4e,0x43,0x6d,0xe8,
  0xe1,0x86,0x31,0x85,0x28,0xe2,0x88,0x24,0x96,0x68,0xe2,0x89,0x28,0xa6,0xa8,0xe2,
  0x8a,0x2c,0xb6,0xe8,0xe2,0x8b,0x30,0xc6,0x28,0xa3,0x89,0x01,0x01,0x00,0x3b
  };

CDispenserProgressBar::CDispenserProgressBar()
{
	m_Properties.append(new CPropertyLayout<FXProgressBar>(this));
	m_Properties.append(new CPropertyFrame<FXProgressBar>(this));
	m_Properties.append(new CPropertyProgressStyle<FXProgressBar>(this));
	m_Properties.append(new CPropertyBackColor<FXProgressBar, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyTextColor<FXProgressBar, FXRGB(0, 0, 255)>(this));
	m_Properties.append(new CPropertyBarColor<FXProgressBar, FXRGB(0, 0, 255)>(this));
	m_Properties.append(new CPropertyBarBackColor<FXProgressBar, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyTextAltColor<FXProgressBar, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyPadTop<FXProgressBar>(this));
	m_Properties.append(new CPropertyPadLeft<FXProgressBar>(this));
	m_Properties.append(new CPropertyPadBottom<FXProgressBar>(this));
	m_Properties.append(new CPropertyPadRight<FXProgressBar>(this));
	m_Properties.append(new CPropertyLeft<FXProgressBar>(this));
	m_Properties.append(new CPropertyTop<FXProgressBar>(this));
	m_Properties.append(new CPropertyWidth<FXProgressBar>(this));
	m_Properties.append(new CPropertyHeight<FXProgressBar>(this));
}

const char* CDispenserProgressBar::getWidgetName(void)
{
	return "FXProgressBar";
}

const void* CDispenserProgressBar::getWidgetIcon(void)
{
	return Progress;
}

const char* CDispenserProgressBar::getWidgetCategory(void)
{
	return "Extended";
}

const int CDispenserProgressBar::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserProgressBar::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserProgressBar::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserProgressBar::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserProgressBar::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserProgressBar::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserProgressBar::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserProgressBar::getClassInfo(void)
{
	return &FXProgressBar::metaClass;
}

FXbool CDispenserProgressBar::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserProgressBar::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserProgressBar::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserProgressBar::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXProgressBar::metaClass);
}

IDispenser* CDispenserProgressBar::clone(void)
{
	return new CDispenserProgressBar;
}

FXWindow* CDispenserProgressBar::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXProgressBar(p, tgt, sel,
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserProgressBar::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXProgressBar(" << parent << ", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream  << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
