/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserTextField.h"

#include <CPropertyEditable.h>
#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertySelBackColor.h>
#include <CPropertySelTextColor.h>
#include <CPropertyText.h>
#include <CPropertyTipText.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyJustify.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyColumns.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyTextStyle : public CProperty
{
public:
	CPropertyTextStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	virtual void set(FXWindow* pWindow, FXString strValue);
	virtual FXString get(FXWindow* pWindow);
	virtual void validate(IConnector* pConnector, FXString& strNewValue);
	virtual void validate(IConnector* pConnector) {}

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

// From FXTextField.h
template <class T>
OPTION CPropertyTextStyle<T>::Style[] = 
{
	{"TEXTFIELD_PASSWD"  , TEXTFIELD_PASSWD  , "Password mode"},
	{"TEXTFIELD_INTEGER" , TEXTFIELD_INTEGER , "Integer mode"},
	{"TEXTFIELD_REAL"    , TEXTFIELD_REAL    , "Real mode"},
	{"TEXTFIELD_READONLY", TEXTFIELD_READONLY, "NOT editable"},
	{0, 0, 0}
};


template <class T>
const char* CPropertyTextStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyTextStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyTextStyle<T>::description(void)
{
	return "Sets the text style for this object";
}

template <class T>
const char* CPropertyTextStyle<T>::tooltip(void)
{
	return "getTextStyle()\nsetTextStyle()";
}

template <class T>
void CPropertyTextStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setTextStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyTextStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getTextStyle() );
}

template <class T>
void CPropertyTextStyle<T>::validate(IConnector* pConnector, FXString& strNewValue)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nOldValue = p->getTextStyle();
	FXint nNewValue = getIntFromString(strNewValue);

	// Has user turned on TEXTFIELD_PASSWD?
	if ( !(nOldValue & TEXTFIELD_PASSWD) && (nNewValue & TEXTFIELD_PASSWD))
	{
		// Yes. So turn off TEXTFIELD_INTEGER|TEXTFIELD_REAL
		nNewValue &= ~(TEXTFIELD_INTEGER|TEXTFIELD_REAL);
	}

	// Has user turned on TEXTFIELD_INTEGER?
	if ( !(nOldValue & TEXTFIELD_INTEGER) && (nNewValue & TEXTFIELD_INTEGER) )
	{
		// Yes. So turn off TEXTFIELD_PASSWD|TEXTFIELD_REAL
		nNewValue &= ~(TEXTFIELD_PASSWD|TEXTFIELD_REAL);
	}

	// Has user turned on TEXTFIELD_REAL?
	if ( !(nOldValue & TEXTFIELD_REAL) && (nNewValue & TEXTFIELD_REAL) )
	{
		// Yes. So turn off TEXTFIELD_PASSWD|TEXTFIELD_INTEGER
		nNewValue &= ~(TEXTFIELD_PASSWD|TEXTFIELD_INTEGER);
	}

	strNewValue = getStringFromInt(nNewValue);
}

template <class T>
OPTION* CPropertyTextStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyTextStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getTextStyle() == 0 )
		return;
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char TextField[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xce,0xce,0xce,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x3a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x90,0xde,0x81,0x9d,0xd9,0x40,0xef,0xff,0xc0,0xa0,0x70,0x48,0xfc,
  0x11,0x72,0xb5,0x00,0x80,0xc0,0x6c,0x3a,0x9f,0xd0,0xa8,0x74,0xca,0x1c,0x1c,0x75,
  0x18,0x1e,0x75,0xcb,0xed,0x12,0xac,0xc8,0x8c,0x40,0xe9,0x2d,0x9b,0x99,0x82,0xe3,
  0x58,0x4c,0x3e,0xbb,0xb7,0xe9,0xc0,0x3a,0xdb,0x7e,0xdb,0xa1,0x60,0xec,0x45,0x7b,
  0xef,0x37,0xf3,0x49,0x4b,0x7e,0x7e,0x80,0x3b,0x75,0x83,0x76,0x85,0x74,0x82,0x88,
  0x89,0x57,0x81,0x8d,0x77,0x8a,0x7b,0x87,0x91,0x66,0x93,0x16,0x7c,0x96,0x67,0x98,
  0x15,0x9a,0x9b,0x65,0x9d,0x14,0x9f,0xa0,0x5d,0xa2,0x13,0xa4,0xa5,0x5b,0xa7,0x12,
  0xa9,0xaa,0x53,0xac,0x48,0x8c,0xaf,0xab,0x8f,0x86,0xb3,0x8d,0x3e,0x5e,0xb1,0xae,
  0x5c,0x4b,0xb8,0x4e,0xc0,0xb0,0xb6,0x8b,0xc1,0x4f,0xc2,0x04,0xbf,0x53,0xc8,0x51,
  0xbc,0x87,0xb8,0xc8,0xca,0x52,0xcc,0x78,0xc4,0x94,0xb3,0xd0,0x4c,0x46,0xc9,0xdc,
  0xc9,0xba,0xda,0xa1,0xd6,0x99,0xcf,0xc7,0xdd,0xe0,0xca,0xd8,0xe0,0xbb,0xe2,0x9e,
  0xcf,0x41,0xe6,0xdc,0xe8,0xc6,0xf0,0xb5,0x61,0xc5,0x4d,0xd9,0xef,0xf2,0xf8,0xea,
  0xa6,0xec,0xa3,0xe4,0x8c,0x01,0xdb,0xa7,0x8e,0xda,0x13,0x67,0xe9,0x04,0x2a,0x94,
  0x56,0x30,0x9c,0xbd,0x6b,0xf3,0xf8,0x79,0xeb,0x71,0x4e,0xdb,0x37,0x7a,0x54,0x10,
  0x82,0xba,0x58,0x4f,0xcf,0x38,0x83,0x6e,0xb4,0xaa,0xfc,0x43,0x55,0x29,0x64,0xb3,
  0x91,0xad,0x4a,0x9a,0x3c,0x88,0x52,0xd6,0xca,0x8c,0x2d,0x7b,0xbd,0xfc,0x12,0x53,
  0xe5,0x4c,0x9a,0x0f,0x3f,0xde,0x3c,0x99,0xb3,0x1d,0xc8,0x90,0x1a,0x77,0xb2,0xec,
  0x09,0xf0,0x27,0xad,0xa0,0x42,0xff,0xd4,0x34,0xfa,0x0a,0x69,0x52,0x9c,0x1e,0x7d,
  0x3e,0x55,0x4a,0x94,0x24,0x53,0x55,0x4e,0x93,0xc6,0x1a,0x73,0xb5,0x54,0x9c,0x39,
  0x10,0xa7,0x42,0x0d,0x34,0xa3,0xac,0xd9,0xb3,0x2b,0x6a,0x8a,0x75,0x52,0x35,0x25,
  0x0e,0x14,0x51,0x47,0xa1,0x9d,0x4b,0x37,0xc6,0xdb,0xbb,0x78,0xf3,0xea,0xdd,0xcb,
  0xb7,0xaf,0xdf,0xbf,0x11,0x00,0x00,0x3b
  };

CDispenserTextField::CDispenserTextField()
{
	m_Properties.append(new CPropertyLayout<FXTextField>(this));
	m_Properties.append(new CPropertyFrame<FXTextField>(this));
	m_Properties.append(new CPropertyJustify<FXTextField>(this));
	m_Properties.append(new CPropertyTextStyle<FXTextField>(this));
//	m_Properties.append(new CPropertyEditable<FXTextField>(this));
	m_Properties.append(new CPropertyBackColor<FXTextField, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyTextColor<FXTextField, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertySelBackColor<FXTextField>(this));
	m_Properties.append(new CPropertySelTextColor<FXTextField>(this));
	m_Properties.append(new CPropertyText<FXTextField>(this));
	m_Properties.append(new CPropertyTipText<FXTextField>(this));
	m_Properties.append(new CPropertyColumns<FXTextField, 10>(this));
	m_Properties.append(new CPropertyPadTop<FXTextField>(this));
	m_Properties.append(new CPropertyPadLeft<FXTextField>(this));
	m_Properties.append(new CPropertyPadBottom<FXTextField>(this));
	m_Properties.append(new CPropertyPadRight<FXTextField>(this));
	m_Properties.append(new CPropertyLeft<FXTextField>(this));
	m_Properties.append(new CPropertyTop<FXTextField>(this));
	m_Properties.append(new CPropertyWidth<FXTextField>(this));
	m_Properties.append(new CPropertyHeight<FXTextField>(this));
}

const char* CDispenserTextField::getWidgetName(void)
{
	return "FXTextField";
}

const void* CDispenserTextField::getWidgetIcon(void)
{
	return TextField;
}

const char* CDispenserTextField::getWidgetCategory(void)
{
	return "Standard";
}

const int CDispenserTextField::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserTextField::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserTextField::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserTextField::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserTextField::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserTextField::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserTextField::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserTextField::getClassInfo(void)
{
	return &FXTextField::metaClass;
}

FXbool CDispenserTextField::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserTextField::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserTextField::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserTextField::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXTextField::metaClass);
}

IDispenser* CDispenserTextField::clone(void)
{
	return new CDispenserTextField;
}

FXWindow* CDispenserTextField::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXTextField(p, 10, tgt, sel, FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserTextField::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXTextField(" << parent << ", 10, 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream  << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
